# EVE Universe IDs
EVE_CATEGORY_ID_STRUCTURE = 65
EVE_GROUP_ID_CONTROL_TOWER = 365
EVE_GROUP_ID_MOBILE_DEPOT = 1246
EVE_GROUP_ID_REFINERY = 1406
EVE_TYPE_ID_CUSTOMS_OFFICE = 2233
EVE_TYPE_ID_IHUB = 32458
EVE_TYPE_ID_TCU = 32226
