{% load i18n %}
{% load humanize %}

<div class="table-responsive">
    <table class="table table-striped table_timers" id="tab_timers_{{ tab_name }}" style="width:100%;">
        <thead>
            <tr>
                <th>{% translate "Eve Time" %}</th>
                <th>{% translate "Local Time" %}</th>
                <th>{% translate "Location" %}</th>
                <th>{% translate "Dist. *" %}</th>
                <th>{% translate "Structure" %} / {% translate "Timer Type" %}</th>
                <th>{% translate "Objective" %} / {% translate "Tags" %}</th>
                <th>{% translate "Name" %} / {% translate "Owner" %}</th>
                <th></th>
            </tr>
        </thead>
    </table>
</div>
